﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Web.Services;
using System.Web.Services.Protocols;

namespace SOAP_Events
{
    class Events
    {
        static void Main(string[] args)
        {
            ttsoapcgi svr = null;
            long cookie = 0;

            try
            {
                svr = new ttsoapcgi();

                CProject[] projs = svr.getProjectList("SOAPUser", "myPassword");
                cookie = svr.ProjectLogon(projs[3], "SOAPUser", "myPassword");

                Console.WriteLine("Logged into {0}", projs[3].database.name);

                // Create a test case
                long tcID = CreateTestCase(svr, cookie);

                // Generate some test runs from that test case
                CItemToTrack[] testRuns = GenerateTestRuns(svr, cookie, tcID);

                // Run the automated tests
                long failedID = RunTests(svr, cookie, testRuns);
                if (failedID != 0)
                {
                    // Create a defect for the failed test run
                    CreateDefectForFailedTestRun(svr, cookie, failedID);
                }
            }
            catch (SoapException e)
            {
                Console.WriteLine("SOAP Fault: {0} - {1}", e.Detail.InnerText, e.Message);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }
            finally
            {
                try
                {
                    if (svr != null)
                        svr.DatabaseLogoff(cookie);
                }
                catch (SoapException e)
                {
                    Console.WriteLine("SOAP Fault: {0} - {1}", e.Detail.InnerText, e.Message);
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.ToString());
                }
            }
        }

        private static void CreateDefectForFailedTestRun(ttsoapcgi svr, long cookie, long trID)
        {
            Console.WriteLine("Creating defect from test run");
            // Create a defect from the test run
            CDefect dft = svr.createDefectForTestRun(cookie, trID);
            dft.type = "Automated Test Failure";
            svr.addDefect(cookie, dft);
        }

        private static long RunTests(ttsoapcgi svr, long cookie, CItemToTrack[] testRuns)
        {
           long nFailedTestRunID = 0;
           if (testRuns != null)
           {
              // Retrieve the test run for edit
              CTestRun tr = svr.editTestRunByRecordID(cookie, testRuns[0].recordid, false);
              
              // Add an estimate event
              CEvent trEst = new CEvent();
              trEst.name = "Estimate";
              trEst.hours = 4.0;
              trEst.hoursSpecified = true;
              trEst.user = "User, SOAP";
              trEst.date = System.DateTime.Now;
              
              // Add start event and begin
              CEvent trStart = new CEvent();
              trStart.name = "Start";
              trStart.user = "User, SOAP";
              trStart.date = System.DateTime.Now;
              
              /*
                 * Do automated test steps here
              */
              
              // Mark the first step as complete, steps are on newlines prefixed with *
              // A step is marked as completed when [Y] is added to the end of the step
              int stepStart = tr.steps.IndexOf('*');
              if (stepStart >= 0)
              {
                 int stepEnd = tr.steps.IndexOf('\n', stepStart);
              
                 if (stepEnd < 0)
                    tr.steps += "[Y]";
                 else
                    tr.steps = tr.steps.Insert(stepEnd, "[Y]");
              }
              
              // Add an enter work event
              CEvent trWork = new CEvent();
              trWork.name = "Enter Work";
              trWork.user = "User, SOAP";
              trWork.date = System.DateTime.Now;
              trWork.hours = trStart.date.Subtract(trWork.date).TotalHours;
              trWork.notes = "First step completed by SOAP script";
              
              // Enter Work has two decimal custom fields, Percent Complete and Remaining Time
              CDecimalField perComp = new CDecimalField();
              perComp.name = "Percent Complete";
              perComp.value = 12.5;
              CDecimalField remainTime = new CDecimalField();
              remainTime.name = "Remaining Time";
              double t = trEst.hours - trWork.hours;
              remainTime.value = t < 0.0 ? 0.0 : t;
              trWork.fieldlist = new CField[] { perComp, remainTime };
              
              /*
                 * Some other steps here one of which fails
              */
              
              // Add a problem statement
              // Problem statements being with ! on a separate line
              tr.steps += "\n!Automated SOAP test failed with return code 2";
              
              // Fail the test run
              CEvent trFail = new CEvent();
              trFail.name = "Fail";
              trFail.user = "User, SOAP";
              trFail.date = System.DateTime.Now;
              
              tr.eventlist = new CEvent[4] { trEst, trStart, trWork, trFail };
              
              Console.WriteLine("Saving test run");
              // Save the test run
              svr.saveTestRun(cookie, tr);
              
              nFailedTestRunID = tr.recordid;
              
           }

           // Return the failed test run record ID
           return nFailedTestRunID;
        }

        private static CItemToTrack[] GenerateTestRuns(ttsoapcgi svr, long cookie, long tcID)
        {
           CItemToTrack[] testRuns = null;

           CTestRunVariantField browser = new CTestRunVariantField();
           browser.name = "Browser";
           CFieldValue ie = new CFieldValue();
           ie.value = "Internet Explorer";
           CFieldValue firefox = new CFieldValue();
           firefox.value = "Firefox";
           CFieldValue opera = new CFieldValue();
           opera.value = "Opera";
           browser.values = new CFieldValue[3] { ie, firefox, opera };
           
           CTestRunVariantField clientType = new CTestRunVariantField();
           clientType.name = "Client Type";
           CFieldValue webClient = new CFieldValue();
           webClient.value = "Web Client";
           clientType.values = new CFieldValue[1] { webClient };
           
           CTestRunVariantField os = new CTestRunVariantField();
           os.name = "Operating System";
           CFieldValue windows = new CFieldValue();
           windows.value = "Windows";
           CFieldValue mac = new CFieldValue();
           mac.value = "Mac";
           CFieldValue linux = new CFieldValue();
           linux.value = "Linux";
           os.values = new CFieldValue[3] { windows, mac, linux };
           
           CTestRunVariantField[] variants = new CTestRunVariantField[3] { browser, clientType, os };
           
           Console.WriteLine("Generating test runs");
           testRuns = svr.generateTestRuns(cookie, tcID, variants, "Alpha Automated Regression");
           Console.WriteLine("Returned {0} test runs", testRuns.Length);
           
           return testRuns;
        }

        private static long CreateTestCase(ttsoapcgi svr, long cookie)
        {
           long nTCRecordID = 0;
           
           CTestCase tc = new CTestCase();
           tc.summary = "[SOAP] Automated Login Regression Test";
           
           tc.isautomated = true;
           tc.isautomatedSpecified = true;
           
           tc.type = "Regression";
           
           // The estimated run time to complete this test
           tc.estimatedtime = "02:30:00";
           
           /* 
              * Fill in the test case steps
              * Line beginning with * is a step
              * Line beginning with + is an expected result
              * Line beginning with # is a comment
           */
           tc.steps = "*Launch the %Z_BROWSER% browser\n"
              + "*Navigate to http://testing.mydomain.com/login.html\n"
              + "+Verify login dialog opens\n"
              + "#If login dialog is not shown, make sure auto-login is disabled and re-run test\n"
              + "*Click add server connection button\n"
              + "+Verify Add Server Connection dialog opens\n"
              + "*Remove any existing connections by pressing Delete\n"
              + "#Intentionally do not show \"Are you sure you want to delete...\" prompt in this case\n"
              + "*Add connection to internal test server\n"
              + "*Press OK button\n"
              + "+Connection information is saved and returned to the login screen\n"
              + "*Login with the wrong user and password\n"
              + "+Verify the login fails with an error\n"
              + "*Login with correct username and no password\n"
              + "+Verify the login fails with an error\n"
              + "*Login with the correct username and password\n"
              + "+Verify logged in successfully without any error dialogs\n"
              + "#Warning dialog may appear if password has expired, this is normal";
           
           // Fill in the custom fields
           CStringField preConditions = new CStringField();
           preConditions.name = "Pre-Conditions";
           preConditions.value = "Application server must be installed.\n"
              + "Valid license keys added and assigned to testing user.";
           tc.customFieldList = new CField[1] { preConditions };
           
           // This test only makes sense on the web client
           CTestCaseVariantField var = new CTestCaseVariantField();
           var.type = "Included";
           var.name = "Client Type";
           var.values = new CFieldValue[1];
           var.values[0] = new CFieldValue();
           var.values[0].value = "Web Client";
           tc.testVariants = new CTestCaseVariantField[1] { var };
           
           // Fill in Ready event
           CEvent readyEvt = new CEvent();
           readyEvt.name = "Ready";
           readyEvt.notes = "Approved for creating test runs";
           readyEvt.user = "User, SOAP";
           readyEvt.date = System.DateTime.Now;
           tc.eventlist = new CEvent[1] { readyEvt };
           
           nTCRecordID = svr.addTestCase(cookie, tc);

           return nTCRecordID;
        }
    }
}
