﻿using System;
using System.Text;
using System.Text.RegularExpressions;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.IO;
using System.Diagnostics;
using System.Collections;
using System.Collections.Generic;

namespace SOAP_Folders_PostSave
{
   class Folder_Management
   {
      static void Main(string[] args)
      {
         // This will stream all output to the event log
         StringWriter log_output = null;
         try
         {
            log_output = new StringWriter();
         }
         catch (IOException e)
         {
            Console.WriteLine("{0} - Cannot create log output.", e.Message);
         }
         
         Console.SetOut(log_output);

         ttsoapcgi svr = null;
         long cookie = 0;

         try
         {
            svr = new ttsoapcgi();

            CProject[] projs = svr.getProjectList("SOAPUser", "myPassword");
            cookie = svr.ProjectLogon(projs[0], "SOAPUser", "myPassword");

            // Pull the record ID of the item that was changed or added from the environment
            // DFSA == Defect State Attribute (Open or Closed)
            bool bIsClosed = Environment.GetEnvironmentVariable("DFSA") == "Closed";
            long id = long.Parse(Environment.GetEnvironmentVariable("RECORDID"));

            // Retrieve the defect and any folders it was in
            CFolder[] flds = svr.getFolderListForEntityByRecordID(cookie, "Defect", id);
            CDefect dfct = svr.getDefectByRecordID(cookie, id, false);

            // Comb over the summary and any found by descriptions looking for hash tags
            List<string> tags = GetHashTagsFromString(dfct.summary);
            foreach (CReportedByRecord reportedBy in dfct.reportedbylist)
               tags.AddRange(GetHashTagsFromString(reportedBy.comments));
            
            // Check each folder and see if there is already tag for it
            List<CFolder> xtraFlds = null;
            if (!bIsClosed)
            {
               xtraFlds = new List<CFolder>();
               foreach (CFolder f in flds)
               {
                  if (f.ispublic)
                  {
                     if (!tags.Contains("#" + f.name))
                        xtraFlds.Add(f);
                     else
                        tags.Remove("#" + f.name);
                  }
               }
            }
            else
            {
               xtraFlds = new List<CFolder>(flds);
            }
            
            // Remove the defect from any extraneous folders
            foreach (CFolder f in xtraFlds)
            {
               CFolderItem item = new CFolderItem();
               item.entityrecordid = dfct.recordid;
               item.entitytablename = "Defect";
            
               svr.removeEntityFromFolderByRecordID(cookie, f.recordid, true, item);
            
               // Check if that particular folder has anything left in it
               if (svr.getEntityListForFolderByRecordID(cookie, f.recordid, true).Length == 0)
               {
                  svr.deleteFolderByRecordID(cookie, f.recordid, true);
                  Console.WriteLine("Deleted folder with record ID {0}", f.recordid);
               }
            }
            
            // Add the defect to folders with the remaining hashtag names
            if (!bIsClosed)
            {
               List<string> paths = GetListOfPublicFolderPaths(svr, cookie);
               // Add to any new folders
               foreach (string tag in tags)
               {
                  // Determine if the folder already exists
                  string folderName = "/Public/" + tag.Substring(1);
                  CFolder addToFolder = null;
                  if (paths.Contains(folderName))
                  {
                     addToFolder = svr.getFolder(cookie, folderName);
                  }
                  else
                  {
                     // Folder doesn't already exist
                     addToFolder = new CFolder();
                     addToFolder.foldertypename = "Folder";
                     addToFolder.name = tag.Substring(1);
                     addToFolder.path = "/Public/" + tag.Substring(1);
                     addToFolder.recordid = svr.addFolder(cookie, addToFolder);
                     Console.WriteLine("Added folder with record ID {0}", addToFolder.recordid);
                  }
            
                  svr.addEntityToFolderByRecordID(cookie, addToFolder.recordid, true, dfct.recordid, "Defect");
               }
            }
            
         }
         catch (SoapException e)
         {
            Console.WriteLine("SOAP Fault: {0} - {1}", e.Code, e.Message);
         }
         catch (Exception e)
         {
            Console.WriteLine(e.ToString());
         }
         finally
         {
            try
            {
               if (svr != null)
                  svr.DatabaseLogoff(cookie);
            }
            catch (SoapException e)
            {
               Console.WriteLine("SOAP Fault: {0} - {1}", e.Code, e.Message);
            }
            catch (Exception e)
            {
               Console.WriteLine(e.ToString());
            }
         }

         log_output.Close();
         
         string sOutput = log_output.ToString();
         string sSource = "TestTrack Folder Demo";
         
         if (sOutput.Length > 0)
         {
            if (!EventLog.SourceExists(sSource))
               EventLog.CreateEventSource(sSource, "Application");
         
            EventLog.WriteEntry(sSource, log_output.ToString());
         }
      }

      private static List<string> GetListOfPublicFolderPaths(ttsoapcgi svr, long cookie)
      {
         CTableColumn[] col = new CTableColumn[1];
         col[0] = new CTableColumn();
         col[0].name = "Path";
      
         CRecordListSoap recLst = svr.getRecordListForTable(cookie, "Folder", null, col);
      
         List<string> paths = new List<string>();
      
         if (recLst.records != null)
         {
            foreach (CRecordRowSoap record in recLst.records)
               paths.Add(record.row[0].value);
         }
      
         return paths;
      }

      private static List<string> GetHashTagsFromString(string str)
      {
         List<string> tags = new List<string>();
      
         if (str != null)
         {
            Regex exp = new Regex(@"#\w+\b");
            MatchCollection matches = exp.Matches(str);
      
            foreach (Match match in matches)
            {
               tags.Add(str.Substring(match.Index, match.Length));
            }
         }
      
         return tags;
      }
   }
}
